@@ -46,6 +46,11 @@ class Logger:
         from .visualization import save_image
         save_image(img, save_path)
 
+    def save_any(self, name, callback):
+        save_path: Path = self.img_dir / name
+        if not save_path.parent.exists():
+            save_path.parent.mkdir(parents=True)
+        callback(save_path)
 
 
 def TextLogger(name, save_dir, handlers=['console', 'file'], verbosity='debug'):
@@ -85,7 +90,9 @@ class Handler:
         handler = logging.handlers.RotatingFileHandler(filename, maxBytes=10485760,
                                                        backupCount=20, encoding='utf-8')
         handler.setLevel(level)
-        formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
+        formatter = logging.Formatter('%(asctime)s - %(levelname)s - %(message)s',
+                                       "%Y-%m-%d %H:%M:%S")
+        # formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
         handler.setFormatter(formatter)
         return handler
 
