@@ -1,4 +1,4 @@
 from .module import Module, SMSOModule
 from .engine import Engine
 from .entry import run, run_lazy
-from .config import basic_args
\ No newline at end of file
+from .config import basic_args, Args
\ No newline at end of file
