@@ -8,8 +8,12 @@ from qqdm import qqdm
 from colorama import Fore, init
 
 from torchlight.logging.logger import Logger
+
 from .module import Module
-from .util import Timer, MetricTracker, PerformanceMonitor, CheckpointCleaner, text_divider, format_num, action_confirm
+from .util import (
+    Timer, MetricTracker, PerformanceMonitor, CheckpointCleaner,
+    text_divider, format_num, action_confirm
+)
 
 init(autoreset=True)
 
@@ -72,21 +76,26 @@ class Engine:
     def set_debug_mode(self):
         self.debug_mode = True
 
-    def resume(self, model_name, base_dir=None):
-        ckpt_dir = self.experiment.ckpt_dir if base_dir is None else Experiment(base_dir).ckpt_dir
-        resume_path = ckpt_dir / 'model-{}.pth'.format(model_name)
-        assert resume_path.exists(), '{} not found'.format(resume_path)
+    def resume(self, ckpt):
+        from contextlib import suppress
+        if os.path.exists(ckpt):
+            with suppress(Exception):
+                self._resume_checkpoint(ckpt)
+                return self
+
+        ckpt_dir = self.experiment.ckpt_dir
+        resume_path = ckpt_dir / 'model-{}.pth'.format(ckpt)
         self._resume_checkpoint(resume_path)
         return self
 
-    def train(self, train_loader, valid_loader=None):
+    def train(self, train_loader, valid_loader=None, **kwargs):
         """
         Full training logic
         """
         for epoch in range(self.start_epoch, self.cfg.max_epochs + 1):
             self._show_divider('=', text='Epoch[{}]'.format(epoch))
             try:
-                self._train(train_loader, epoch, valid_loader)
+                self._train(train_loader, epoch, valid_loader, **kwargs)
             except KeyboardInterrupt:
                 msg = Fore.RED + "Oops!" + Fore.RESET + " Training was terminated by Ctrl-C. "
                 "Do you want to save the latest checkpoint?"
@@ -101,9 +110,9 @@ class Engine:
                 break
 
     # TODO: use this function to support mannuly control training dataloader outside of engine
-    def _train(self, train_loader, epoch, valid_loader=None):
+    def _train(self, train_loader, epoch, valid_loader=None, **kwargs):
         timer.tic()
-        result = self._train_epoch(epoch, train_loader)
+        result = self._train_epoch(epoch, train_loader, **kwargs)
         self.epoch = epoch
 
         # ------------------ Save logged informations into log dict ------------------ #
@@ -113,7 +122,7 @@ class Engine:
         # ------------------ Print logged informations to the screen ----------------- #
         if valid_loader is not None and (epoch % self.cfg.valid_per_epoch == 0 or self.debug_mode):
             val_log = self._valid_epoch(epoch, valid_loader)
-            log.update(**{'val_'+k: v for k, v in val_log.items()})
+            log.update(**{'val_' + k: v for k, v in val_log.items()})
 
         if self.debug_mode:
             return
@@ -139,14 +148,14 @@ class Engine:
 
     def test(self, test_loader, name=None):
         old_logger = self.logger
-        self.test_log_dir = self.experiment.save_dir / 'test' / 'epoch{}'.format(self.epoch-1)
+        self.test_log_dir = self.experiment.save_dir / 'test' / 'epoch{}'.format(self.epoch - 1)
         if name is not None:
             self.test_log_dir = self.test_log_dir / name
         self.test_log_dir.mkdir(parents=True, exist_ok=True)
         self.logger = Logger(self.test_log_dir)
 
         val_log = self._valid_epoch(1, test_loader)
-        log = {'test_'+k: v for k, v in val_log.items()}
+        log = {'test_' + k: v for k, v in val_log.items()}
 
         self._log_log(log)
 
@@ -158,7 +167,7 @@ class Engine:
             self.logger.info('    {:15s}: {}'.format(str(key), value))
         self._show_divider('-')
 
-    def _train_epoch(self, epoch, train_loader):
+    def _train_epoch(self, epoch, train_loader, **kwargs):
         """
         Training logic for an epoch
 
@@ -166,16 +175,17 @@ class Engine:
         :return: A log that contains average loss and met
         ric in this epoch.
         """
-
-        self.module.on_epoch_start(train=True)
         
         metric_tracker = MetricTracker()
         len_epoch = len(train_loader)
+        
+        self.module.on_epoch_start(train=True, epoch=epoch, loader=train_loader, **kwargs)
+        
         pbar = self._progress_bar(total=len_epoch)
         pbar.set_description('Train [{}|{}]'.format(epoch, self.cfg.max_epochs))
         for batch_idx, data in enumerate(train_loader):
             gstep = (epoch - 1) * len_epoch + batch_idx + 1
-            results = self.module.step(data, train=True, epoch=epoch, step=gstep)
+            results = self.module.step(data, train=True, epoch=epoch, step=gstep, **kwargs)
 
             self.logger.tensorboard.set_step(gstep, mode='train')
             for name, value in results.metrics.items():
@@ -202,7 +212,7 @@ class Engine:
                 break
 
         pbar.close()
-        self.module.on_epoch_end(train=True)
+        self.module.on_epoch_end(train=True, epoch=epoch, loader=train_loader, **kwargs)
 
         return metric_tracker.result()
 
@@ -216,11 +226,12 @@ class Engine:
         import torch
 
         metric_tracker = MetricTracker()
-
         len_epoch = len(valid_loader)
+        
+        self.module.on_epoch_start(train=False, epoch=epoch, loader=valid_loader)
+        
         pbar = self._progress_bar(total=len_epoch)
         pbar.set_description('Valid [{}|{}]'.format(epoch, self.cfg.max_epochs))
-        self.module.on_epoch_start(train=False)
         with torch.no_grad():
             len_epoch = len(valid_loader)
             for batch_idx, data in enumerate(valid_loader):
@@ -249,7 +260,7 @@ class Engine:
                 if self.debug_mode:
                     break
             pbar.close()
-        self.module.on_epoch_end(train=False)
+        self.module.on_epoch_end(train=False, epoch=epoch, loader=valid_loader)
         return metric_tracker.result()
 
     def _save_checkpoint(self, epoch, postfix=None):
@@ -284,7 +295,7 @@ class Engine:
         resume_path = str(resume_path)
         self.logger.info("Loading checkpoint: {} ...".format(resume_path))
         checkpoint = torch.load(resume_path)
-        
+
         if 'epoch' in checkpoint:
             self.start_epoch = checkpoint['epoch'] + 1
         else:
@@ -298,7 +309,7 @@ class Engine:
 
         self.module.load_state_dict(checkpoint['module'])
 
-        self.logger.info("Checkpoint loaded. Resume from epoch {}".format(self.start_epoch-1))
+        self.logger.info("Checkpoint loaded. Resume from epoch {}".format(self.start_epoch - 1))
 
     def _progress_bar(self, total):
         if self.cfg.pbar == 'qqdm':
@@ -317,6 +328,6 @@ class Engine:
 
 
 def _progress(batch_idx, loader):
-    current = batch_idx * loader.batch_size
-    total = len(loader) * loader.batch_size
+    current = batch_idx
+    total = len(loader)
     return '[{}/{} ({:.0f}%)]'.format(current, total, 100.0 * current / total)
